package uk.ac.bath.oy213.ip.server.commands.net;

import uk.ac.bath.oy213.ip.server.Server;
import uk.ac.bath.oy213.ip.server.net.ConnectionThread;
import uk.ac.bath.oy213.ip.server.net.Packet;
import uk.ac.bath.oy213.ip.server.security.PasswordHash;
import uk.ac.bath.oy213.ip.server.util.LoggingLevel;

import java.security.NoSuchAlgorithmException;
import java.security.spec.InvalidKeySpecException;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.UUID;

/**
 * Created by Ollie on 13/04/15.
 */
public class AuthenticateUserCommand extends NetCommand {

    public AuthenticateUserCommand() {

        super("login");

        setRequiresAuthentication(false);

    }

    @Override
    public void dispatch(ConnectionThread connection, String[] args, Object data) {

        dispatch(connection,args);

    }

    @Override
    public void dispatch(ConnectionThread connection, String[] args) {

        if (args.length != 2) {

            Packet packet = new Packet(null,"login-fail",null,null);

            connection.sendPacket(packet);
            return;

        }

        try {

            ResultSet resultSet = Server.instance().getDatabaseConnection().callStoredProcedure("getPassword", args[0].trim().toLowerCase());

            String hash;

            if (!resultSet.next()) {

                connection.sendPacket(new Packet(null,"login-fail",null,null));
                return;

            } else {

                hash = resultSet.getString("password");

            }

            if (PasswordHash.validatePassword(args[1], hash)) {

                UUID tokenID = Server.instance().getTokenRegistry().assignToken(args[0]);

                connection.setUsername(args[0].trim().toLowerCase());

                Packet packet = new Packet(tokenID,"login-success",null,null);

                connection.sendPacket(packet);

            } else {

                Packet packet = new Packet(null,"login-fail",null,null);

                connection.sendPacket(packet);

            }

        } catch (SQLException | NoSuchAlgorithmException e) {

            Server.instance().log(LoggingLevel.SEVERE, e.getMessage());

            connection.sendPacket(new Packet(null,"error",null,null));

        } catch (InvalidKeySpecException e) {

            Server.instance().log(LoggingLevel.WARNING, e.getMessage());

            Packet packet = new Packet(null,"login-fail",null,null);

            connection.sendPacket(packet);

        }

    }

}
