package uk.ac.bath.oy213.ip.server.commands.net;

import uk.ac.bath.oy213.ip.server.Server;
import uk.ac.bath.oy213.ip.server.net.ConnectionThread;
import uk.ac.bath.oy213.ip.server.net.Packet;
import uk.ac.bath.oy213.ip.server.security.PasswordHash;
import uk.ac.bath.oy213.ip.server.util.LoggingLevel;

import java.security.NoSuchAlgorithmException;
import java.security.spec.InvalidKeySpecException;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
 * Created by Ollie on 13/04/15.
 */
public class CreateUserCommand extends NetCommand {

    public CreateUserCommand() {

        super("create-user");

        setRequiresAuthentication(false);

    }

    @Override
    public void dispatch(ConnectionThread connection, String[] args, Object data) {

        dispatch(connection, args);

    }

    @Override
    public void dispatch(ConnectionThread connection, String[] args) {

        if (args.length != 2) {

            connection.sendPacket(new Packet(null,"error",null,null));

        } else {

            ResultSet resultSet = Server.instance().getDatabaseConnection().callStoredProcedure("getUsersByUsername", args[0].trim().toLowerCase());

            try {

                if (!resultSet.next()) {

                    Server.instance().getDatabaseConnection().callStoredProcedure("createUser", args[0].trim().toLowerCase(), PasswordHash.createHash(args[1]));

                } else {

                    connection.sendPacket(new Packet(null, "creation-failed", new String[]{"username taken"}, null));
                    return;

                }

            } catch (SQLException | InvalidKeySpecException | NoSuchAlgorithmException e) {

                Server.instance().log(LoggingLevel.SEVERE, e.getMessage());

                connection.sendPacket(new Packet(null,"error",null,null));
                return;

            }

            connection.setUsername(args[0]);

            connection.sendPacket(new Packet(Server.instance().getTokenRegistry().assignToken(args[0]), "creation-success", null, null));

        }

    }

}
