package uk.ac.bath.oy213.ip.server.commands.net;

import ip.e_biz.Card;
import uk.ac.bath.oy213.ip.server.Server;
import uk.ac.bath.oy213.ip.server.net.ConnectionThread;
import uk.ac.bath.oy213.ip.server.net.Packet;
import uk.ac.bath.oy213.ip.server.util.CardBuilder;

import java.io.File;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
 * Created by Ollie on 27/04/15.
 */
public class RemoveCardCommand extends NetCommand {

    public RemoveCardCommand() {

        super("remove-card");

    }

    @Override
    public void dispatch(ConnectionThread connection, String[] args, Object data) {

        dispatch(connection,args);

    }

    @Override
    public void dispatch(ConnectionThread connection, String[] args) {

        if (args.length != 1) {

            connection.sendPacket(new Packet(null,"error",null,null));

        } else {

            ResultSet resultSet = Server.instance().getDatabaseConnection().callStoredProcedure("getUsersOwnCards",connection.getUsername());

            try {

                int cardID = Integer.parseInt(args[0]);

                if (!resultSet.next()) {

                    deleteSavedCard(connection, cardID);

                } else {

                    do {

                        Card c = CardBuilder.buildCard(resultSet);

                        if (c.getID() == cardID) {

                            File frontImg = new File(resultSet.getString("frontImageLocation"));
                            File backImg = new File(resultSet.getString("backImageLocation"));

                            if (!frontImg.delete() || !backImg.delete()) {

                                connection.sendPacket(new Packet(null,"remove-failed",null,null));
                                return;

                            }

                            Server.instance().getDatabaseConnection().callStoredProcedure("deleteCard", cardID);

                            connection.sendPacket(new Packet(null,"remove-success",null,null));
                            return;

                        }

                    } while (resultSet.next());

                    deleteSavedCard(connection, cardID);

                }

            } catch (SQLException | NumberFormatException | NullPointerException e) {

                connection.sendPacket(new Packet(null,"error",null,null));

            }

        }

    }

    private void deleteSavedCard(ConnectionThread connection, int cardID) {

        ResultSet resultSet = Server.instance().getDatabaseConnection().callStoredProcedure("getUsersSavedCards",connection.getUsername());

        try {

            if (!resultSet.next()) {

                connection.sendPacket(new Packet(null, "remove-failed", null, null));

            } else {

                do {

                    int savedCardID = resultSet.getInt("cardID");

                    if (savedCardID == cardID) {

                        Server.instance().getDatabaseConnection().callStoredProcedure("deleteSavedCard", connection.getUsername(), cardID);
                        connection.sendPacket(new Packet(null, "remove-success", null, null));
                        return;

                    }

                } while (resultSet.next());

                connection.sendPacket(new Packet(null, "remove-failed", null, null));

            }

        } catch (SQLException e) {

            connection.sendPacket(new Packet(null,"error",null,null));

        }

    }

}
