package uk.ac.bath.oy213.ip.server.commands.net;

import ip.e_biz.Card;
import uk.ac.bath.oy213.ip.server.Server;
import uk.ac.bath.oy213.ip.server.net.ConnectionThread;
import uk.ac.bath.oy213.ip.server.net.Packet;
import uk.ac.bath.oy213.ip.server.util.CardBuilder;
import uk.ac.bath.oy213.ip.server.util.FileHandler;

import java.io.File;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
 * Created by Ollie on 16/04/15.
 */
public class UpdateCardCommand extends NetCommand {

    public UpdateCardCommand() {

        super("update-card");

    }

    @Override
    public void dispatch(ConnectionThread connection, String[] args, Object data) {

        Card card;

        try {

            card = (Card) data;

        } catch (NullPointerException | ClassCastException e) {

            connection.sendPacket(new Packet(null,"error",null,null));
            return;

        }

        ResultSet resultSet = Server.instance().getDatabaseConnection().callStoredProcedure("getUsersOwnCards",connection.getUsername());

        try {

            if (!resultSet.next()) {

                connection.sendPacket(new Packet(null,"update-card-failed",null,null));

            } else {

                do {

                    Card c = CardBuilder.buildCard(resultSet);

                    if (c.getID() == card.getID()) {

                        File frontImg = new File(resultSet.getString("frontImageLocation"));
                        File backImg = new File(resultSet.getString("backImageLocation"));

                        if (!frontImg.delete() || !backImg.delete()) {

                            connection.sendPacket(new Packet(null,"update-card-failed",null,null));
                            return;

                        }

                        FileHandler.writeImage(card.getBackImage(),resultSet.getString("backImageLocation"));
                        FileHandler.writeImage(card.getFrontImage(),resultSet.getString("frontImageLocation"));

                        Server.instance().getDatabaseConnection().callStoredProcedure("updateCard", card.getID(), resultSet.getString("frontImageLocation"), resultSet.getString("backImageLocation"), card.getEmail(), card.getTelephone(), card.getWebsite(),false,false);

                        connection.sendPacket(new Packet(null,"update-card-success",null,null));
                        return;

                    }

                } while (resultSet.next());

                connection.sendPacket(new Packet(null,"update-card-failed",null,null));

            }

        } catch (SQLException | NumberFormatException | NullPointerException e) {

            connection.sendPacket(new Packet(null,"error",null,null));

        }

    }

    @Override
    public void dispatch(ConnectionThread connection, String[] args) {

        connection.sendPacket(new Packet(null,"error",null,null));

    }

}
