package uk.ac.bath.oy213.ip.server.commands.user;

import uk.ac.bath.oy213.ip.server.Server;
import uk.ac.bath.oy213.ip.server.util.Colour;
import uk.ac.bath.oy213.ip.server.util.LoggingLevel;

/**
 * Created by Ollie on 28/03/15.
 */
public class HelpCommand extends UserCommand {

    private int leftGap = 7+2*(Colour.FORMATTING_LENGTH);

    public HelpCommand() {

        super("help");

    }

    @Override
    public String[] showHelp(boolean isVerbose) {

        return isVerbose ? new String[] {"provides detailed help for a particular command.","help <command>"} : new String[] {"shows this screen."};

    }

    @Override
    public void dispatch(String[] args) {

        if (args.length == 0) { //print non-verbose help for all commands

            Server.instance().log(LoggingLevel.SERVER, "Showing all commands:\n");

            for (UserCommand c : Server.instance().getUserCommands()) {

                for (int i = 0; i < c.showHelp(false).length; i++) {

                    if (i == 0) {

                        System.out.printf("%" + leftGap + "s: %-30s%n", Colour.ANSI_GREEN + c.getCanonincalAlias() + Colour.ANSI_RESET, c.showHelp(false)[i]);

                    } else {

                        System.out.printf("%" + leftGap + "s %-30s%n", Colour.ANSI_RESET+Colour.ANSI_RESET, c.showHelp(false)[i]);

                    }

                }

            }

        } else {

            for (UserCommand c : Server.instance().getUserCommands()) { //check to see if said command exists

                if (c.hasAlias(args[0])) { //command exists, print the verbose help

                    Server.instance().log(LoggingLevel.SERVER, "Showing help for '" + Colour.ANSI_GREEN + args[0] + LoggingLevel.SERVER.getReset() + "':\n");

                    for (int i = 0; i < c.showHelp(true).length; i++) {

                        if (i == 0) {

                            System.out.printf("%" + leftGap + "s: %-30s%n", Colour.ANSI_GREEN + c.getCanonincalAlias() + Colour.ANSI_RESET, c.showHelp(true)[i]);

                        } else {

                            System.out.printf("%" + leftGap + "s %-30s%n", Colour.ANSI_RESET+Colour.ANSI_RESET, c.showHelp(true)[i]);

                        }

                    }

                    for (int i = 0; i < c.getAliases().size(); i++) {

                        if (i == 0) {

                            System.out.printf("%n%" + leftGap + "s: %-30s%n", Colour.ANSI_GREEN + "aliases" + Colour.ANSI_RESET, c.getAliases().get(i));

                        } else {

                            System.out.printf("%" + leftGap + "s %-30s%n", Colour.ANSI_RESET + Colour.ANSI_RESET, c.getAliases().get(i));

                        }

                    }

                    return;

                }

            }

            //notify the user that no such command was found
            Server.instance().log(LoggingLevel.WARNING, "Command '" + Colour.ANSI_GREEN + args[0] + LoggingLevel.WARNING.getReset() + "' not found.");

        }

    }

}
