package uk.ac.bath.oy213.ip.server.location;

import java.util.ArrayList;

/**
 * Created by Ollie on 17/04/15.
 */
public class LocationTable {

    ArrayList[][] table;

    public LocationTable() {

        table = new ArrayList[10][10];

        for (int lat = 0; lat < 10; lat++) {

            for (int lon = 0; lon < 10; lon++) {

                table[lat][lon] = new ArrayList();

            }

        }

    }

    public void updateLocation(UserLocation userLocation) {

        int latBucket = (int) userLocation.getLatitude() - 49;
        int lonBucket = (int) userLocation.getLongitude() + 8;

        ArrayList<UserLocation> bucket = (ArrayList<UserLocation>) table[latBucket][lonBucket];

        if (!bucket.contains(userLocation)) {

            removeUser(userLocation.getUsername());

        }

        bucket.add(userLocation);

    }

    public ArrayList<String> getUsersWithinRadius(UserLocation userLocation, double radiusInKM) {

        ArrayList<String> nearbyUsers = new ArrayList<>();

        int latBucket = (int) userLocation.getLatitude() - 49;
        int lonBucket = (int) userLocation.getLongitude() + 8;

        ArrayList<UserLocation> bucket = (ArrayList<UserLocation>) table[latBucket][lonBucket];

        for (UserLocation location: bucket) {

            if (!location.getUsername().equalsIgnoreCase(userLocation.getUsername()) && distanceInKM(userLocation.getLatitude(), userLocation.getLongitude(), location.getLatitude(), location.getLongitude()) <= radiusInKM) {

                nearbyUsers.add(location.getUsername());

            }

        }

        return nearbyUsers;

    }

    public void removeUser(String username) {

        if (username != null) {

            ArrayList<UserLocation> toRemove = new ArrayList<>();

            for (int lat = 0; lat < 10; lat++) {

                for (int lon = 0; lon < 10; lon++) {

                    for (UserLocation location : (ArrayList<UserLocation>) table[lat][lon]) {

                        if (location.getUsername().equals(username)) {

                            toRemove.add(location);

                        }

                    }

                    for (UserLocation location : toRemove) {

                        table[lat][lon].remove(location);

                    }

                    toRemove.clear();

                }

            }

        }

    }

    private double distanceInKM(double lat1, double lng1, double lat2, double lng2) {

        double earthRadius = 6371;

        double dLat = Math.toRadians(lat2-lat1);
        double dLng = Math.toRadians(lng2 - lng1);

        double sindLat = Math.sin(dLat / 2);
        double sindLng = Math.sin(dLng / 2);

        double a = Math.pow(sindLat, 2) + Math.pow(sindLng, 2)
                * Math.cos(Math.toRadians(lat1)) * Math.cos(Math.toRadians(lat2));

        double c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1-a));

        return earthRadius * c;

    }

}
