package uk.ac.bath.oy213.ip.server.net;

import uk.ac.bath.oy213.ip.server.Server;
import uk.ac.bath.oy213.ip.server.commands.net.NetCommand;
import uk.ac.bath.oy213.ip.server.util.Colour;
import uk.ac.bath.oy213.ip.server.util.LoggingLevel;

import java.io.*;
import java.net.Socket;

/**
 * Created by Ollie on 28/03/15.
 */
public class ConnectionThread extends Thread {

    private Socket socket;
    private ObjectInputStream in;
    private ObjectOutputStream out;
    private String username;
    private boolean active;
    private final int connectionID;

    public ConnectionThread(Socket socket, int connectionID) {

        active = false;

        this.connectionID = connectionID;
        this.socket = socket;
        Server.instance().log(LoggingLevel.INFO, "client connecting on "+Colour.ANSI_GREEN+socket.getInetAddress().toString()+LoggingLevel.INFO.getReset()+".");

        try {

            out = new ObjectOutputStream(new BufferedOutputStream(socket.getOutputStream()));
            out.flush();

            in = new ObjectInputStream(new BufferedInputStream(socket.getInputStream()));

            Server.instance().log(LoggingLevel.INFO, "connection established from "+Colour.ANSI_GREEN+socket.getInetAddress().toString()+LoggingLevel.INFO.getReset()+" ("+connectionID+").");

        } catch (IOException e) {

            Server.instance().log(LoggingLevel.SEVERE, e.getMessage());

            try {

                socket.close();

            } catch (IOException e1) {

                //swallow

            }

            Server.instance().getListenerThread().removeThread(this);

        }

    }

    @Override
    public void run() {

        active = true;

        Packet packet;

        do {

            try {

                packet = Packet.uncompress(in.readObject());

                Server.instance().log(LoggingLevel.INFO, "received "+ Colour.ANSI_GREEN+packet.getCommand()+LoggingLevel.INFO.getReset()+" from "+connectionID+" ("+username+").");

                for (NetCommand c: Server.instance().getNetCommands()) {

                    if (c.hasAlias(packet.getCommand())) {

                        if (c.doesRequireAuthentication()) {

                            if (username == null || !Server.instance().getTokenRegistry().verifyToken(packet.getAuthToken(),username)) {

                                interrupt();
                                continue;

                            }

                        }

                        if (packet.getData() == null) {

                            c.dispatch(this, packet.getArgs());

                        } else {

                            c.dispatch(this, packet.getArgs(), packet.getData());

                        }

                    }

                }

            } catch (ClassCastException | ClassNotFoundException e) {

                Server.instance().log(LoggingLevel.INFO, "received malformed packet from client "+socket.getInetAddress().toString()+" disconnecting them...");
                interrupt();

            } catch (IOException | NullPointerException e) {

                interrupt();

            }

        } while(active);

        Server.instance().log(LoggingLevel.INFO, "connection from "+socket.getInetAddress().toString()+" disconnected.");

        Server.instance().getLocationTable().removeUser(username);

        Server.instance().getListenerThread().removeThread(this);

    }

    @Override
    public void interrupt() {

        sendPacket(new Packet(null,"session-timeout",null,null));

        Server.instance().getLocationTable().removeUser(username);

        active = false;

    }

    public String getUsername() {

        return username;

    }

    public void setUsername(String username) {

        this.username = username;

    }

    public int getConnectionID() {

        return connectionID;

    }

    public String getAddress() {

        return socket.getInetAddress().getHostAddress();

    }

    public void sendPacket(Packet packet) {

        try {

            Server.instance().log(LoggingLevel.INFO, "sent "+ Colour.ANSI_GREEN+packet.getCommand()+LoggingLevel.INFO.getReset()+" to "+connectionID+" ("+username+").");

            out.writeObject(Packet.compress(packet));
            out.flush();

        } catch (IOException e) {

            Server.instance().log(LoggingLevel.WARNING, e.getMessage());

        } catch (NullPointerException e) {

            //swallow - stream never set up

        }

        if (packet.getCommand().equals("error")) {

            Server.instance().getLocationTable().removeUser(username);

        }

    }

}
