package uk.ac.bath.oy213.ip.server.security;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.UUID;

/**
 * Created by Ollie on 13/04/15.
 */
public class TokenRegistry {

    private Hashtable<UUID,AuthenticationToken> idToTokenMap;
    private Hashtable<AuthenticationToken,String> tokenToUserMap;
    private Hashtable<String,AuthenticationToken> userToTokenMap;

    private static long LEASE_TIME = 1800000;

    public TokenRegistry() {

        idToTokenMap = new Hashtable<>();
        tokenToUserMap = new Hashtable<>();
        userToTokenMap = new Hashtable<>();

    }

    public UUID assignToken(String username) {

        if (userToTokenMap.get(username) != null) {

            AuthenticationToken token = userToTokenMap.get(username);

            idToTokenMap.remove(token.getId());
            tokenToUserMap.remove(token);
            userToTokenMap.remove(username);

        }

        UUID id = UUID.randomUUID();

        AuthenticationToken token = new AuthenticationToken(id, System.currentTimeMillis()+LEASE_TIME);

        idToTokenMap.put(id,token);
        tokenToUserMap.put(token, username);
        userToTokenMap.put(username, token);

        return id;

    }

    public boolean verifyToken(UUID tokenID, String username) {

        if (tokenID != null && username != null && idToTokenMap.get(tokenID) != null && tokenToUserMap.get(idToTokenMap.get(tokenID)).equals(username)) {

            if (idToTokenMap.get(tokenID).getExpirationTime() > System.currentTimeMillis()) {

                idToTokenMap.get(tokenID).setExpirationTime(System.currentTimeMillis()+LEASE_TIME);

                return true;

            }

        }

        return false;

    }

    public void revokeToken(String username) {

        AuthenticationToken token = userToTokenMap.get(username);

        idToTokenMap.remove(token.getId());
        tokenToUserMap.remove(token);
        userToTokenMap.remove(username);

    }

    public ArrayList<AuthenticationToken> getTokens() {

        return new ArrayList<>(tokenToUserMap.keySet());

    }

    public String getUsername(AuthenticationToken token) {

        return tokenToUserMap.get(token);

    }

}
