package uk.ac.bath.oy213.ip.server.util;

import ip.e_biz.Card;
import uk.ac.bath.oy213.ip.server.Server;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;

/**
 * Created by Ollie on 17/04/15.
 */
public class CardBuilder {

    public static Card buildCard(ResultSet rs) {

        try {

            BufferedImage frontImage = ImageIO.read(new File(rs.getString("frontImageLocation")));
            BufferedImage backImage = ImageIO.read(new File(rs.getString("backImageLocation")));
            String email = rs.getString("email");
            String telephone = rs.getString("telephone");
            String website = rs.getString("website");

            Card card = new Card(frontImage, backImage, email, telephone, website);
            card.setID(rs.getInt("cardID"));
            card.setShared(rs.getBoolean("isShared"));

            return card;

        } catch (SQLException |IOException e) {

            e.printStackTrace();

            Server.instance().log(LoggingLevel.SEVERE, e.getMessage());
            return null;

        }

    }

    public static ArrayList<Card> getSharedCardsFromUser(String username) {

        ArrayList<Card> cards = new ArrayList<>();

        ResultSet rs = Server.instance().getDatabaseConnection().callStoredProcedure("getUsersSharedCards",username);

        try {

            while (rs.next()) {

                cards.add(buildCard(rs));

            }

        } catch (SQLException e) {

            Server.instance().log(LoggingLevel.SEVERE, e.getMessage());

        }

        return cards;

    }

    public static ArrayList<Integer> getSharedCardIDsFromUser(String username) {

        ArrayList<Integer> cards = new ArrayList<>();

        ResultSet rs = Server.instance().getDatabaseConnection().callStoredProcedure("getUsersSharedCards",username);

        try {

            while (rs.next()) {

                cards.add(rs.getInt("cardID"));

            }

        } catch (SQLException e) {

            Server.instance().log(LoggingLevel.SEVERE, e.getMessage());

        }

        return cards;

    }

}
